@@ -1,7 +1,7 @@
 ///
 /// @file  EratBig.hpp
 ///
-/// Copyright (C) 2020 Kim Walisch, <kim.walisch@gmail.com>
+/// Copyright (C) 2022 Kim Walisch, <kim.walisch@gmail.com>
 ///
 /// This file is distributed under the BSD License. See the COPYING
 /// file in the top level directory.
@@ -27,15 +27,15 @@ namespace primesieve {
 class EratBig : public Wheel210_t
 {
 public:
-  void init(uint64_t, uint64_t, uint64_t);
+  void init(uint64_t, uint64_t, uint64_t, MemoryPool&);
   NOINLINE void crossOff(uint8_t*);
   bool enabled() const { return enabled_; }
 private:
   uint64_t maxPrime_ = 0;
   uint64_t log2SieveSize_ = 0;
   uint64_t moduloSieveSize_ = 0;
+  MemoryPool* memoryPool_ = nullptr;
   std::vector<SievingPrime*> buckets_;
-  MemoryPool memoryPool_;
   bool enabled_ = false;
   void storeSievingPrime(uint64_t, uint64_t, uint64_t);
   NOINLINE void crossOff(uint8_t*, Bucket*);
