@@ -1,9 +1,24 @@
 ///
 /// @file   CpuInfo.cpp
-/// @brief  Get detailed information about the CPU's caches
-///         on Windows, macOS and Linux.
+/// @brief  Get detailed information about the CPU's caches.
+///         Ideally each primesieve thread should use a sieve array
+///         size that corresponds to the cache sizes of the CPU core
+///         the thread is currently running on. Unfortunately, due to
+///         the many different operating systems, compilers and CPU
+///         architectures this is difficult to implement (portably).
+///         Furthermore, any thread may randomly be moved to another
+///         CPU core by the operating system scheduler.
 ///
-/// Copyright (C) 2020 Kim Walisch, <kim.walisch@gmail.com>
+///         Hence in order to ensure good scaling we use the following
+///         alternative strategy: We detect the cache sizes of one of
+///         the CPU cores at startup and all primesieve threads use a
+///         sieve array size that is related to that CPU core's cache
+///         sizes. For homogeneous CPUs with just one type of CPU core
+///         this strategy is optimal. For hybrid CPUs with multiple
+///         types of CPU cores we try to detect the cache sizes of the
+///         CPU core type that e.g. occurs most frequently.
+///
+/// Copyright (C) 2022 Kim Walisch, <kim.walisch@gmail.com>
 ///
 /// This file is distributed under the BSD License. See the COPYING
 /// file in the top level directory.
@@ -17,6 +32,8 @@
 #include <string>
 #include <vector>
 
+using std::size_t;
+
 #if defined(__APPLE__)
   #if !defined(__has_include)
     #define APPLE_SYSCTL
@@ -27,7 +44,11 @@
 
 #if defined(_WIN32)
 
+#include <primesieve/pmath.hpp>
+
 #include <windows.h>
+#include <iterator>
+#include <map>
 
 #if defined(__i386__) || \
     defined(_M_IX86) || \
@@ -40,7 +61,6 @@
 #if defined(IS_X86)
 
 #include <algorithm>
-#include <iterator>
 
 // Check if compiler supports <intrin.h>
 #if defined(__has_include)
@@ -60,8 +80,6 @@
   #define GNUC_CPUID
 #endif
 
-using namespace std;
-
 namespace {
 
 /// CPUID is not portable across all x86 CPU vendors and there
@@ -115,14 +133,14 @@ void cpuId(int cpuInfo[4], int eax)
 /// Remove all leading and trailing
 /// space characters.
 ///
-void trimString(string& str)
+void trimString(std::string& str)
 {
-  string spaceChars = " \f\n\r\t\v";
+  std::string spaceChars = " \f\n\r\t\v";
   size_t pos = str.find_first_not_of(spaceChars);
   str.erase(0, pos);
 
   pos = str.find_last_not_of(spaceChars);
-  if (pos != string::npos)
+  if (pos != std::string::npos)
     str.erase(pos + 1);
 }
 
@@ -130,13 +148,11 @@ void trimString(string& str)
 
 #endif
 
-using namespace std;
-
 namespace {
 
-string getCpuName()
+std::string getCpuName()
 {
-  string cpuName;
+  std::string cpuName;
 
 #if defined(IS_X86)
   // Get the CPU name using CPUID.
@@ -145,19 +161,19 @@ string getCpuName()
 
   int cpuInfo[4] = { 0, 0, 0, 0 };
   cpuId(cpuInfo, 0x80000000);
-  vector<int> vect;
+  std::vector<int> vect;
 
   // check if CPU name is supported
   if ((unsigned) cpuInfo[0] >= 0x80000004u)
   {
     cpuId(cpuInfo, 0x80000002);
-    copy_n(cpuInfo, 4, back_inserter(vect));
+    std::copy_n(cpuInfo, 4, std::back_inserter(vect));
 
     cpuId(cpuInfo, 0x80000003);
-    copy_n(cpuInfo, 4, back_inserter(vect));
+    std::copy_n(cpuInfo, 4, std::back_inserter(vect));
 
     cpuId(cpuInfo, 0x80000004);
-    copy_n(cpuInfo, 4, back_inserter(vect));
+    std::copy_n(cpuInfo, 4, std::back_inserter(vect));
 
     vect.push_back(0);
     cpuName = (char*) vect.data();
@@ -177,81 +193,147 @@ void CpuInfo::init()
 // Windows 7 (2009) or later
 #if _WIN32_WINNT >= 0x0601
 
-  using LPFN_GLPIEX = decltype(&GetLogicalProcessorInformationEx);
+  using LPFN_GAPC = decltype(&GetActiveProcessorCount);
+  LPFN_GAPC getCpuCoreCount = (LPFN_GAPC) (void*) GetProcAddress(
+      GetModuleHandle(TEXT("kernel32")), "GetActiveProcessorCount");
 
+  if (getCpuCoreCount)
+    logicalCpuCores_ = getCpuCoreCount(ALL_PROCESSOR_GROUPS);
+
+  using LPFN_GLPIEX = decltype(&GetLogicalProcessorInformationEx);
   LPFN_GLPIEX glpiex = (LPFN_GLPIEX) (void*) GetProcAddress(
-      GetModuleHandle(TEXT("kernel32")),
-      "GetLogicalProcessorInformationEx");
+      GetModuleHandle(TEXT("kernel32")), "GetLogicalProcessorInformationEx");
 
   if (!glpiex)
     return;
 
   DWORD bytes = 0;
-  glpiex(RelationAll, 0, &bytes);
+  glpiex(RelationCache, 0, &bytes);
 
   if (!bytes)
     return;
 
-  vector<char> buffer(bytes);
-  SYSTEM_LOGICAL_PROCESSOR_INFORMATION_EX* info;
+  std::vector<char> buffer(bytes);
 
-  if (!glpiex(RelationAll, (SYSTEM_LOGICAL_PROCESSOR_INFORMATION_EX*) &buffer[0], &bytes))
+  if (!glpiex(RelationCache, (SYSTEM_LOGICAL_PROCESSOR_INFORMATION_EX*) &buffer[0], &bytes))
     return;
 
+  struct CpuCoreCacheInfo
+  {
+    CpuCoreCacheInfo() :
+      cacheSizes{0, 0, 0, 0},
+      cacheSharing{0, 0, 0, 0}
+    { }
+    std::array<size_t, 4> cacheSizes;
+    std::array<size_t, 4> cacheSharing;
+  };
+
+  struct L1CacheStatistics
+  {
+    long cpuCoreId = -1;
+    size_t cpuCoreCount = 0;
+  };
+
+  using CacheSize_t = size_t;
+  // Items must be sorted in ascending order
+  std::map<CacheSize_t, L1CacheStatistics> l1CacheStatistics;
+  std::vector<CpuCoreCacheInfo> cacheInfo;
+  std::size_t totalL1CpuCores = 0;
+  SYSTEM_LOGICAL_PROCESSOR_INFORMATION_EX* info;
+
+  // Fill the cacheInfo vector with the L1, L2 & L3 cache
+  // sizes and cache sharing of each CPU core.
   for (size_t i = 0; i < bytes; i += info->Size)
   {
     info = (SYSTEM_LOGICAL_PROCESSOR_INFORMATION_EX*) &buffer[i];
 
     if (info->Relationship == RelationCache &&
-        info->Cache.GroupMask.Group == 0 &&
         info->Cache.Level >= 1 &&
         info->Cache.Level <= 3 &&
         (info->Cache.Type == CacheData ||
          info->Cache.Type == CacheUnified))
     {
-      auto level = info->Cache.Level;
-      cacheSizes_[level] = info->Cache.CacheSize;
-
-      // @warning: GetLogicalProcessorInformationEx() reports
-      // incorrect data when Windows is run inside a virtual
-      // machine. Specifically the GROUP_AFFINITY.Mask will
-      // only have 1 or 2 bits set for each CPU cache even if
-      // more logical CPU cores share the cache
-      auto mask = info->Cache.GroupMask.Mask;
-      cacheSharing_[level] = 0;
+      size_t cpuCoreIndex = 0;
+      size_t cacheSharing = 0;
 
       // Cache.GroupMask.Mask contains one bit set for
-      // each logical CPU core sharing the cache
+      // each logical CPU core sharing the cache.
+      for (auto mask = info->Cache.GroupMask.Mask; mask > 0; mask &= mask - 1)
+        cacheSharing++;
+
+      auto cacheSize = info->Cache.CacheSize;
+      auto level = info->Cache.Level;
+      auto processorGroup = info->Cache.GroupMask.Group;
+      using Mask_t = decltype(info->Cache.GroupMask.Mask);
+      auto maxCpusPerProcessorGroup = numberOfBits<Mask_t>();
+      auto mask = info->Cache.GroupMask.Mask;
+      mask = (mask == 0) ? 1 : mask;
+
       for (; mask > 0; mask &= mask - 1)
-        cacheSharing_[level]++;
-    }
-
-    if (info->Relationship == RelationProcessorCore)
-    {
-      cpuCores_++;
-      threadsPerCore_ = 0;
-      size_t size = info->Processor.GroupCount;
-
-      for (size_t j = 0; j < size; j++)
       {
-        // Processor.GroupMask.Mask contains one bit set for
-        // each thread associated to the current CPU core
-        auto mask = info->Processor.GroupMask[j].Mask;
-        for (; mask > 0; mask &= mask - 1)
-          threadsPerCore_++;
-      }
+        // Convert next 1 bit into cpuCoreIndex,
+        // by counting trailing zeros.
+        while (!(mask & (((Mask_t) 1) << cpuCoreIndex)))
+          cpuCoreIndex++;
 
-      cpuThreads_ += threadsPerCore_;
+        // Note that calculating the cpuCoreId this way is not
+        // 100% correct as processor groups may not be fully
+        // filled (they may have less than maxCpusPerProcessorGroup
+        // CPU cores). However our formula yields unique
+        // cpuCoreIds which is good enough for our usage.
+        size_t cpuCoreId = processorGroup * maxCpusPerProcessorGroup + cpuCoreIndex;
+
+        if (cacheInfo.size() <= cpuCoreId)
+          cacheInfo.resize((cpuCoreId + 1) * 2);
+        cacheInfo[cpuCoreId].cacheSizes[level] = cacheSize;
+        cacheInfo[cpuCoreId].cacheSharing[level] = cacheSharing;
+
+        // Count the number of occurences of each type of L1 cache.
+        // If one of these L1 cache types is used predominantly
+        // we will use that cache as our default cache size.
+        if (level == 1)
+        {
+          auto& mapEntry = l1CacheStatistics[cacheSize];
+          totalL1CpuCores++;
+          mapEntry.cpuCoreCount++;
+          if (mapEntry.cpuCoreId == -1)
+            mapEntry.cpuCoreId = (long) cpuCoreId;
+        }
+      }
     }
   }
+
+  // Check if one of the L1 cache types is used
+  // by more than 80% of all CPU cores.
+  for (const auto& item : l1CacheStatistics)
+  {
+    if (item.second.cpuCoreCount > totalL1CpuCores * 0.80)
+    {
+      long cpuCoreId = item.second.cpuCoreId;
+      cacheSizes_ = cacheInfo[cpuCoreId].cacheSizes;
+      cacheSharing_ = cacheInfo[cpuCoreId].cacheSharing;
+      return;
+    }
+  }
+
+  // For hybrid CPUs with many different L1 cache types
+  // we pick one from the middle that is hopefully
+  // representative for the CPU's overall performance.
+  if (!l1CacheStatistics.empty())
+  {
+    auto iter = l1CacheStatistics.begin();
+    std::advance(iter, (l1CacheStatistics.size() - 1) / 2);
+    long cpuCoreId = iter->second.cpuCoreId;
+    cacheSizes_ = cacheInfo[cpuCoreId].cacheSizes;
+    cacheSharing_ = cacheInfo[cpuCoreId].cacheSharing;
+  }
+
 // Windows XP or later
 #elif _WIN32_WINNT >= 0x0501
 
   using LPFN_GLPI = decltype(&GetLogicalProcessorInformation);
-
   LPFN_GLPI glpi = (LPFN_GLPI) (void*) GetProcAddress(
-      GetModuleHandle(TEXT("kernel32")),
-      "GetLogicalProcessorInformation");
+      GetModuleHandle(TEXT("kernel32")), "GetLogicalProcessorInformation");
 
   if (!glpi)
     return;
@@ -262,8 +344,9 @@ void CpuInfo::init()
   if (!bytes)
     return;
 
+  size_t threadsPerCore = 0;
   size_t size = bytes / sizeof(SYSTEM_LOGICAL_PROCESSOR_INFORMATION);
-  vector<SYSTEM_LOGICAL_PROCESSOR_INFORMATION> info(size);
+  std::vector<SYSTEM_LOGICAL_PROCESSOR_INFORMATION> info(size);
 
   if (!glpi(&info[0], &bytes))
     return;
@@ -275,12 +358,11 @@ void CpuInfo::init()
       // ProcessorMask contains one bit set for
       // each logical CPU core related to the
       // current physical CPU core
-      auto mask = info[i].ProcessorMask;
-      for (threadsPerCore_ = 0; mask > 0; threadsPerCore_++)
-        mask &= mask - 1;
+      threadsPerCore = 0;
+      for (auto mask = info[i].ProcessorMask; mask > 0; mask &= mask - 1)
+        threadsPerCore++;
 
-      cpuCores_++;
-      cpuThreads_ += threadsPerCore_;
+      logicalCpuCores_ += threadsPerCore;
     }
   }
 
@@ -297,11 +379,11 @@ void CpuInfo::init()
 
       // We assume the L1 and L2 caches are private
       if (info[i].Cache.Level <= 2)
-        cacheSharing_[level] = threadsPerCore_;
+        cacheSharing_[level] = threadsPerCore;
 
       // We assume the L3 cache is shared
       if (info[i].Cache.Level == 3)
-        cacheSharing_[level] = threadsPerCore_ * cpuCores_;
+        cacheSharing_[level] = logicalCpuCores_;
     }
   }
 
@@ -311,7 +393,7 @@ void CpuInfo::init()
   // shared e.g. Intel Core 2 Duo/Quad CPUs and
   // Intel Atom x5-Z8350 CPUs.
   if (hasL2Cache() && !hasL3Cache())
-    cacheSharing_[2] = threadsPerCore_ * cpuCores_;
+    cacheSharing_[2] = logicalCpuCores_;
 #endif
 }
 
@@ -325,8 +407,6 @@ void CpuInfo::init()
 #include <cstddef>
 #include <sys/sysctl.h>
 
-using namespace std;
-
 namespace {
 
 /// Get CPU information from the operating
@@ -334,15 +414,15 @@ namespace {
 /// https://www.freebsd.org/cgi/man.cgi?sysctl(3)
 ///
 template <typename T>
-vector<T> getSysctl(const string& name)
+std::vector<T> getSysctl(const std::string& name)
 {
-  vector<T> res;
+  std::vector<T> res;
   size_t bytes = 0;
 
   if (!sysctlbyname(name.data(), 0, &bytes, 0, 0))
   {
     size_t size = ceilDiv(bytes, sizeof(T));
-    vector<T> buffer(size, 0);
+    std::vector<T> buffer(size, 0);
     if (!sysctlbyname(name.data(), buffer.data(), &bytes, 0, 0))
       res = buffer;
   }
@@ -350,9 +430,9 @@ vector<T> getSysctl(const string& name)
   return res;
 }
 
-string getCpuName()
+std::string getCpuName()
 {
-  string cpuName;
+  std::string cpuName;
 
   auto buffer = getSysctl<char>("machdep.cpu.brand_string");
   if (!buffer.empty())
@@ -367,55 +447,19 @@ namespace primesieve {
 
 void CpuInfo::init()
 {
-  auto cpuCores = getSysctl<size_t>("hw.physicalcpu");
-  if (!cpuCores.empty())
-    cpuCores_ = cpuCores[0];
-
-  auto cpuThreads = getSysctl<size_t>("hw.logicalcpu");
-  if (!cpuThreads.empty())
-    cpuThreads_ = cpuThreads[0];
-
-  if (hasCpuCores() && hasCpuThreads())
-    threadsPerCore_ = cpuThreads_ / cpuCores_;
+  auto logicalCpuCores = getSysctl<size_t>("hw.logicalcpu");
+  if (!logicalCpuCores.empty())
+    logicalCpuCores_ = logicalCpuCores[0];
 
   // https://developer.apple.com/library/content/releasenotes/Performance/RN-AffinityAPI/index.html
   auto cacheSizes = getSysctl<size_t>("hw.cachesize");
-  for (size_t i = 1; i < min(cacheSizes.size(), cacheSizes_.size()); i++)
+  for (size_t i = 1; i < std::min(cacheSizes.size(), cacheSizes_.size()); i++)
     cacheSizes_[i] = cacheSizes[i];
 
   // https://developer.apple.com/library/content/releasenotes/Performance/RN-AffinityAPI/index.html
   auto cacheConfig = getSysctl<size_t>("hw.cacheconfig");
-  for (size_t i = 1; i < min(cacheConfig.size(), cacheSharing_.size()); i++)
+  for (size_t i = 1; i < std::min(cacheConfig.size(), cacheSharing_.size()); i++)
     cacheSharing_[i] = cacheConfig[i];
-
-#if !(defined(__x86_64__) || \
-      defined(__i386__) || \
-      defined(__ppc64__) || \
-      defined(__ppc__))
-
-  // For Apple silicon CPUs (ARM & ARM64) sysctl returns
-  // erroneous L2 cache information. For the L1 cache
-  // sysctl returns the cache size per core which is correct
-  // (like on x86 CPUs). However for the L2 cache sysctl
-  // returns the total cache size. It is unclear whether
-  // hw.cachesize or hw.cacheconfig is erroneous.
-  // See: https://github.com/kimwalisch/primesieve/issues/96
-  //
-  // Because of this sysctl issue it is impossible to know
-  // the exact L2 cache size per core and whether the L2
-  // cache is private or shared. However based on benchmarks
-  // of the Apple M1 CPU the L2 cache scales nicely with
-  // multi-threading provided we use less or equal the
-  // (total L2 cache size / threads sharing the L2 cache).
-  //
-  // For these reasons we make an educated guess here:
-  // If the CPU has an L2 cache we assume it is fast (will
-  // scale nicely with multi-threading) and that using some
-  // of it will improve the sieving performance.
-  //
-  if (hasL2Cache())
-    sysctlL2CacheWorkaround_ = true;
-#endif
 }
 
 } // namespace
@@ -430,7 +474,6 @@ void CpuInfo::init()
 #include <set>
 #include <sstream>
 
-using namespace std;
 using namespace primesieve;
 
 namespace {
@@ -438,21 +481,21 @@ namespace {
 /// Remove all leading and trailing
 /// space characters.
 ///
-void trimString(string& str)
+void trimString(std::string& str)
 {
-  string spaceChars = " \f\n\r\t\v";
+  std::string spaceChars = " \f\n\r\t\v";
   size_t pos = str.find_first_not_of(spaceChars);
   str.erase(0, pos);
 
   pos = str.find_last_not_of(spaceChars);
-  if (pos != string::npos)
+  if (pos != std::string::npos)
     str.erase(pos + 1);
 }
 
-string getString(const string& filename)
+std::string getString(const std::string& filename)
 {
-  ifstream file(filename);
-  string str;
+  std::ifstream file(filename);
+  std::string str;
 
   // Read the first string,
   // stops at any space character
@@ -462,25 +505,25 @@ string getString(const string& filename)
     return {};
 }
 
-size_t getValue(const string& filename)
+size_t getValue(const std::string& filename)
 {
-  string str = getString(filename);
+  std::string str = getString(filename);
   size_t val = 0;
 
   if (!str.empty())
-    val = stoul(str);
+    val = std::stoul(str);
 
   return val;
 }
 
-size_t getCacheSize(const string& filename)
+size_t getCacheSize(const std::string& filename)
 {
-  string str = getString(filename);
+  std::string str = getString(filename);
   size_t val = 0;
 
   if (!str.empty())
   {
-    val = stoul(str);
+    val = std::stoul(str);
     char lastChar = str.back();
 
     // The last character may be:
@@ -505,13 +548,13 @@ size_t getCacheSize(const string& filename)
 /// Returns an empty string if line does
 /// not contain the CPU name.
 ///
-string getCpuName(const string& line)
+std::string getCpuName(const std::string& line)
 {
   // Examples of CPU names:
   // model name : Intel(R) Core(TM) i7-6700 CPU @ 3.40GHz
   // Processor  : ARMv7 Processor rev 5 (v7l)
   // cpu        : POWER9 (raw), altivec supported
-  static set<string> cpuLabels
+  const std::set<std::string> cpuLabels
   {
     "model name",
     "Processor",
@@ -519,11 +562,11 @@ string getCpuName(const string& line)
   };
 
   size_t pos = line.find(':');
-  string cpuName;
+  std::string cpuName;
 
-  if (pos != string::npos)
+  if (pos != std::string::npos)
   {
-    string label = line.substr(0, pos);
+    std::string label = line.substr(0, pos);
     trimString(label);
     if (cpuLabels.find(label) != cpuLabels.end())
       cpuName = line.substr(pos + 1);
@@ -532,29 +575,29 @@ string getCpuName(const string& line)
   return cpuName;
 }
 
-bool isValid(const string& cpuName)
+bool isValid(const std::string& cpuName)
 {
   if (cpuName.empty())
     return false;
-  if (cpuName.find_first_not_of("0123456789") == string::npos)
+  if (cpuName.find_first_not_of("0123456789") == std::string::npos)
     return false;
 
   return true;
 }
 
-string getCpuName()
+std::string getCpuName()
 {
-  ifstream file("/proc/cpuinfo");
-  string notFound;
+  std::ifstream file("/proc/cpuinfo");
+  std::string notFound;
 
   if (file)
   {
-    string line;
+    std::string line;
     size_t i = 0;
 
-    while (getline(file, line))
+    while (std::getline(file, line))
     {
-      string cpuName = getCpuName(line);
+      std::string cpuName = getCpuName(line);
       trimString(cpuName);
 
       if (isValid(cpuName))
@@ -567,14 +610,14 @@ string getCpuName()
   return notFound;
 }
 
-vector<string> split(const string& str,
-                     char delimiter)
+std::vector<std::string> split(const std::string& str,
+                               char delimiter)
 {
-  vector<string> tokens;
-  string token;
-  istringstream tokenStream(str);
+  std::string token;
+  std::vector<std::string> tokens;
+  std::istringstream tokenStream(str);
 
-  while (getline(tokenStream, token, delimiter))
+  while (std::getline(tokenStream, token, delimiter))
     tokens.push_back(token);
 
   return tokens;
@@ -585,7 +628,7 @@ vector<string> split(const string& str,
 /// Example: 0-8,18-26
 /// https://www.kernel.org/doc/Documentation/cputopology.txt
 ///
-size_t parseThreadList(const string& filename)
+size_t parseThreadList(const std::string& filename)
 {
   size_t threads = 0;
   auto threadList = getString(filename);
@@ -598,8 +641,8 @@ size_t parseThreadList(const string& filename)
       threads++;
     else
     {
-      auto t0 = stoul(values.at(0));
-      auto t1 = stoul(values.at(1));
+      auto t0 = std::stoul(values.at(0));
+      auto t1 = std::stoul(values.at(1));
       threads += t1 - t0 + 1;
     }
   }
@@ -613,17 +656,17 @@ size_t parseThreadList(const string& filename)
 /// Example: 00000000,00000000,00000000,07fc01ff
 /// https://www.kernel.org/doc/Documentation/cputopology.txt
 ///
-size_t parseThreadMap(const string& filename)
+size_t parseThreadMap(const std::string& filename)
 {
   size_t threads = 0;
-  string threadMap = getString(filename);
+  std::string threadMap = getString(filename);
 
   for (char c : threadMap)
   {
     if (c != ',')
     {
-      string hexChar { c };
-      size_t bitmap = stoul(hexChar, nullptr, 16);
+      std::string hexChar { c };
+      size_t bitmap = std::stoul(hexChar, nullptr, 16);
       for (; bitmap > 0; threads++)
         bitmap &= bitmap - 1;
     }
@@ -639,8 +682,8 @@ size_t parseThreadMap(const string& filename)
 /// But you cannot know in advance if any of these
 /// files exist, hence you need to try both.
 ///
-size_t getThreads(const string& threadList,
-                  const string& threadMap)
+size_t getThreads(const std::string& threadList,
+                  const std::string& threadMap)
 {
   size_t threads = parseThreadList(threadList);
 
@@ -656,45 +699,35 @@ namespace primesieve {
 
 void CpuInfo::init()
 {
-  string cpusOnline = "/sys/devices/system/cpu/online";
-  cpuThreads_ = parseThreadList(cpusOnline);
+  std::string cpusOnline = "/sys/devices/system/cpu/online";
+  logicalCpuCores_ = parseThreadList(cpusOnline);
 
-  // Works on Linux kernel >= 5.3
-  string threadSiblingsList = "/sys/devices/system/cpu/cpu0/topology/core_cpus_list";
-  string threadSiblings = "/sys/devices/system/cpu/cpu0/topology/core_cpus";
-  threadsPerCore_ = getThreads(threadSiblingsList, threadSiblings);
-
-  if (!threadsPerCore_)
-  {
-    // Works on Linux kernel < 5.3
-    threadSiblingsList = "/sys/devices/system/cpu/cpu0/topology/thread_siblings_list";
-    threadSiblings = "/sys/devices/system/cpu/cpu0/topology/thread_siblings";
-    threadsPerCore_ = getThreads(threadSiblingsList, threadSiblings);
-  }
-
-  if (hasCpuThreads() &&
-      hasThreadsPerCore())
-    cpuCores_ = cpuThreads_ / threadsPerCore_;
+  // For hybrid CPUs with multiple types of CPU cores Linux seems
+  // to order the CPU cores within /sys/devices/system/cpu*
+  // from fastest to slowest. By picking a CPU core from the middle
+  // we hopefully get an average CPU core that is representative
+  // for the CPU's overall (multi-threading) performance.
+  std::string cpuNumber = std::to_string(logicalCpuCores_ / 2);
 
   // Retrieve CPU cache info
   for (size_t i = 0; i <= 3; i++)
   {
-    string path = "/sys/devices/system/cpu/cpu0/cache/index" + to_string(i);
-    string cacheLevel = path + "/level";
+    std::string path = "/sys/devices/system/cpu/cpu" + cpuNumber + "/cache/index" + std::to_string(i);
+    std::string cacheLevel = path + "/level";
     size_t level = getValue(cacheLevel);
 
     if (level >= 1 &&
         level <= 3)
     {
-      string type = path + "/type";
-      string cacheType = getString(type);
+      std::string type = path + "/type";
+      std::string cacheType = getString(type);
 
       if (cacheType == "Data" ||
           cacheType == "Unified")
       {
-        string cacheSize = path + "/size";
-        string sharedCpuList = path + "/shared_cpu_list";
-        string sharedCpuMap = path + "/shared_cpu_map";
+        std::string cacheSize = path + "/size";
+        std::string sharedCpuList = path + "/shared_cpu_list";
+        std::string sharedCpuMap = path + "/shared_cpu_map";
         cacheSizes_[level] = getCacheSize(cacheSize);
         cacheSharing_[level] = getThreads(sharedCpuList, sharedCpuMap);
       }
@@ -712,10 +745,7 @@ namespace primesieve {
 const CpuInfo cpuInfo;
 
 CpuInfo::CpuInfo() :
-  cpuCores_(0),
-  cpuThreads_(0),
-  threadsPerCore_(0),
-  sysctlL2CacheWorkaround_(false),
+  logicalCpuCores_(0),
   cacheSizes_{0, 0, 0, 0},
   cacheSharing_{0, 0, 0, 0}
 {
@@ -723,7 +753,7 @@ CpuInfo::CpuInfo() :
   {
     init();
   }
-  catch (exception& e)
+  catch (std::exception& e)
   {
     // We don't trust the operating system to reliably report
     // all CPU information. In case an unexpected error
@@ -734,7 +764,7 @@ CpuInfo::CpuInfo() :
   }
 }
 
-string CpuInfo::cpuName() const
+std::string CpuInfo::cpuName() const
 {
   try
   {
@@ -744,33 +774,28 @@ string CpuInfo::cpuName() const
     // startup but instead we lazy load it when needed.
     return getCpuName();
   }
-  catch (exception&)
+  catch (std::exception&)
   {
     return {};
   }
 }
 
-size_t CpuInfo::cpuCores() const
+size_t CpuInfo::logicalCpuCores() const
 {
-  return cpuCores_;
+  return logicalCpuCores_;
 }
 
-size_t CpuInfo::cpuThreads() const
-{
-  return cpuThreads_;
-}
-
-size_t CpuInfo::l1CacheSize() const
+size_t CpuInfo::l1CacheBytes() const
 {
   return cacheSizes_[1];
 }
 
-size_t CpuInfo::l2CacheSize() const
+size_t CpuInfo::l2CacheBytes() const
 {
   return cacheSizes_[2];
 }
 
-size_t CpuInfo::l3CacheSize() const
+size_t CpuInfo::l3CacheBytes() const
 {
   return cacheSizes_[3];
 }
@@ -790,12 +815,7 @@ size_t CpuInfo::l3Sharing() const
   return cacheSharing_[3];
 }
 
-size_t CpuInfo::threadsPerCore() const
-{
-  return threadsPerCore_;
-}
-
-string CpuInfo::getError() const
+std::string CpuInfo::getError() const
 {
   return error_;
 }
@@ -805,16 +825,10 @@ bool CpuInfo::hasCpuName() const
   return !cpuName().empty();
 }
 
-bool CpuInfo::hasCpuCores() const
+bool CpuInfo::hasLogicalCpuCores() const
 {
-  return cpuCores_ >= 1 &&
-         cpuCores_ <= (1 << 20);
-}
-
-bool CpuInfo::hasCpuThreads() const
-{
-  return cpuThreads_ >= 1 &&
-         cpuThreads_ <= (1 << 20);
+  return logicalCpuCores_ >= 1 &&
+         logicalCpuCores_ <= (1 << 20);
 }
 
 bool CpuInfo::hasL1Cache() const
@@ -857,23 +871,4 @@ bool CpuInfo::hasL3Sharing() const
          cacheSharing_[3] <= (1 << 20);
 }
 
-bool CpuInfo::hasThreadsPerCore() const
-{
-  return threadsPerCore_ >= 1 &&
-         threadsPerCore_ <= (1 << 10);
-}
-
-bool CpuInfo::hasPrivateL2Cache() const
-{
-  return hasL2Cache() &&
-         hasL2Sharing() &&
-         hasThreadsPerCore() &&
-         l2Sharing() <= threadsPerCore_;
-}
-
-bool CpuInfo::sysctlL2CacheWorkaround() const
-{
-  return sysctlL2CacheWorkaround_;
-}
-
 } // namespace
