@@ -3,7 +3,7 @@
 /// @brief  The Erat class manages prime sieving using the
 ///         EratSmall, EratMedium, EratBig classes.
 ///
-/// Copyright (C) 2020 Kim Walisch, <kim.walisch@gmail.com>
+/// Copyright (C) 2022 Kim Walisch, <kim.walisch@gmail.com>
 ///
 /// This file is distributed under the BSD License. See the COPYING
 /// file in the top level directory.
@@ -24,12 +24,10 @@
 #include <cassert>
 #include <memory>
 
-using namespace std;
-
 namespace {
 
 /// unset bits < start
-const array<uint8_t, 37> unsetSmaller =
+const std::array<uint8_t, 37> unsetSmaller =
 {
   0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xfe, 0xfe, 0xfe, 0xfe, 0xfc, 0xfc, 0xf8, 0xf8,
@@ -39,7 +37,7 @@ const array<uint8_t, 37> unsetSmaller =
 };
 
 /// unset bits > stop
-const array<uint8_t, 37> unsetLarger =
+const std::array<uint8_t, 37> unsetLarger =
 {
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01,
   0x01, 0x01, 0x01, 0x03, 0x03, 0x07, 0x07, 0x07,
@@ -89,7 +87,7 @@ void Erat::init(uint64_t start,
   uint64_t dist = sieveSize_ * 30 + 6;
   segmentLow_ = start_ - rem;
   segmentHigh_ = checkedAdd(segmentLow_, dist);
-  segmentHigh_ = min(segmentHigh_, stop);
+  segmentHigh_ = std::min(segmentHigh_, stop);
 
   initErat();
 }
@@ -97,7 +95,7 @@ void Erat::init(uint64_t start,
 void Erat::initSieve(uint64_t sieveSize)
 {
   sieveSize_ = floorPow2(sieveSize);
-  sieveSize_ = inBetween(8, sieveSize_, 4096);
+  sieveSize_ = inBetween(16, sieveSize_, 4096);
   sieveSize_ *= 1024;
 
   sieve_ = new uint8_t[sieveSize_];
@@ -115,24 +113,21 @@ void Erat::initErat()
   if (sqrtStop > maxPreSieve_)
     eratSmall_.init(stop_, l1CacheSize, maxEratSmall_);
   if (sqrtStop > maxEratSmall_)
-    eratMedium_.init(stop_, sieveSize_, maxEratMedium_);
+    eratMedium_.init(stop_, sieveSize_, maxEratMedium_, memoryPool_);
   if (sqrtStop > maxEratMedium_)
-    eratBig_.init(stop_, sieveSize_, sqrtStop);
+    eratBig_.init(stop_, sieveSize_, sqrtStop, memoryPool_);
 }
 
 /// EratMedium and EratBig usually run fastest using a sieve
-/// size that matches the CPUs L2 cache size. EratSmall
-/// however runs fastest using a sieve size that matches the
-/// CPUs L1 cache size. Hence we use a smaller sieve size
-/// (L1 cache size) in EratSmall and a larger sieve size (L2
-/// cache size) in both EratMedium and EratBig.
+/// size that is slightly smaller than the CPU's L2 cache size.
+/// EratSmall however runs fastest using a sieve size that
+/// matches the CPU's L1 cache size. Hence we use a smaller
+/// sieve size (L1 cache size) in EratSmall and a larger sieve
+/// size (< L2 cache size) in both EratMedium and EratBig.
 ///
 uint64_t Erat::getL1CacheSize() const
 {
-  if (!cpuInfo.hasL1Cache())
-    return sieveSize_;
-
-  uint64_t size = cpuInfo.l1CacheSize();
+  uint64_t size = cpuInfo.hasL1Cache() ? cpuInfo.l1CacheBytes() : config::L1D_CACHE_BYTES;
   uint64_t minSize = 8 << 10;
   uint64_t maxSize = 4096 << 10;
 
@@ -154,18 +149,16 @@ uint64_t Erat::byteRemainder(uint64_t n)
   return n;
 }
 
-/// Pre-sieve multiples of small primes e.g. <= 19
+/// Pre-sieve multiples of small primes < 100
 /// to speed up the sieve of Eratosthenes
 ///
 void Erat::preSieve()
 {
-  preSieve_->copy(sieve_, sieveSize_, segmentLow_);
+  preSieve_->preSieve(sieve_, sieveSize_, segmentLow_);
 
   // unset bits < start
   if (segmentLow_ <= start_)
   {
-    if (start_ <= maxPreSieve_)
-      sieve_[0] = 0xff;
     uint64_t rem = byteRemainder(start_);
     sieve_[0] &= unsetSmaller[rem];
   }
@@ -193,7 +186,7 @@ void Erat::sieveSegment()
     uint64_t dist = sieveSize_ * 30;
     segmentLow_ = checkedAdd(segmentLow_, dist);
     segmentHigh_ = checkedAdd(segmentHigh_, dist);
-    segmentHigh_ = min(segmentHigh_, stop_);
+    segmentHigh_ = std::min(segmentHigh_, stop_);
   }
 }
 
@@ -212,7 +205,7 @@ void Erat::sieveLastSegment()
   // unset bytes > stop
   uint64_t bytes = sieveSize_ % 8;
   bytes = (8 - bytes) % 8;
-  fill_n(&sieve_[sieveSize_], bytes, (uint8_t) 0);
+  std::fill_n(&sieve_[sieveSize_], bytes, (uint8_t) 0);
 
   segmentLow_ = stop_;
 }
