@@ -2,7 +2,7 @@
 /// @file   LookupTables.cpp
 /// @brief  Static gobal arrays.
 ///
-/// Copyright (C) 2020 Kim Walisch, <kim.walisch@gmail.com>
+/// Copyright (C) 2022 Kim Walisch, <kim.walisch@gmail.com>
 ///
 /// This file is distributed under the BSD License. See the COPYING
 /// file in the top level directory.
@@ -25,7 +25,14 @@ namespace primesieve {
 /// 64 bits of the sieve array correspond to 8 bytes which span
 /// an interval of size 30 * 8 = 240.
 ///
-const std::array<uint64_t, 64> bitValues =
+/// The index for this lookup table is computed using the count
+/// trailing zeros CPU instruction. As a special case CTZ may return
+/// the operand size (number of bits) if the input is zero. Hence
+/// the maximum index is 64 for e.g. TZCNT(0) (on x64 CPUs)
+/// therefore we add an additional 0 at the end of the array to
+/// prevent out of bounds acceses.
+///
+const std::array<uint64_t, 65> bitValues =
 {
     7,  11,  13,  17,  19,  23,  29, 31,
    37,  41,  43,  47,  49,  53,  59, 61,
@@ -34,7 +41,8 @@ const std::array<uint64_t, 64> bitValues =
   127, 131, 133, 137, 139, 143, 149, 151,
   157, 161, 163, 167, 169, 173, 179, 181,
   187, 191, 193, 197, 199, 203, 209, 211,
-  217, 221, 223, 227, 229, 233, 239, 241
+  217, 221, 223, 227, 229, 233, 239, 241,
+  0
 };
 
 /// The De Bruijn bitscan is a fast method to compute the index of
