@@ -2,7 +2,7 @@
 /// @file   iterator-c.cpp
 /// @brief  C port of primesieve::iterator.
 ///
-/// Copyright (C) 2019 Kim Walisch, <kim.walisch@gmail.com>
+/// Copyright (C) 2022 Kim Walisch, <kim.walisch@gmail.com>
 ///
 /// This file is distributed under the BSD License. See the COPYING
 /// file in the top level directory.
@@ -16,9 +16,9 @@
 #include <stdint.h>
 #include <cerrno>
 #include <exception>
+#include <iostream>
 #include <vector>
 
-using namespace std;
 using namespace primesieve;
 
 namespace {
@@ -35,9 +35,9 @@ void clearPrimeGenerator(primesieve_iterator* it)
   it->primeGenerator = nullptr;
 }
 
-vector<uint64_t>& getPrimes(primesieve_iterator* it)
+std::vector<uint64_t>& getPrimes(primesieve_iterator* it)
 {
-  using T = vector<uint64_t>;
+  using T = std::vector<uint64_t>;
   T* primes = (T*) it->vector;
   return *primes;
 }
@@ -53,7 +53,7 @@ void primesieve_init(primesieve_iterator* it)
   it->i = 0;
   it->last_idx = 0;
   it->dist = 0;
-  it->vector = new vector<uint64_t>;
+  it->vector = new std::vector<uint64_t>;
   it->primeGenerator = nullptr;
   it->is_error = false;
 }
@@ -68,8 +68,6 @@ void primesieve_skipto(primesieve_iterator* it,
   it->i = 0;
   it->last_idx = 0;
   it->dist = 0;
-  auto& primes = getPrimes(it);
-  primes.clear();
   clearPrimeGenerator(it);
 }
 
@@ -88,24 +86,23 @@ void primesieve_generate_next_primes(primesieve_iterator* it)
 {
   auto& primes = getPrimes(it);
   auto primeGenerator = getPrimeGenerator(it);
+  std::size_t size = 0;
 
   try
   {
-    while (true)
+    while (!size)
     {
       if (!it->primeGenerator)
       {
         IteratorHelper::next(&it->start, &it->stop, it->stop_hint, &it->dist);
         it->primeGenerator = new PrimeGenerator(it->start, it->stop);
         primeGenerator = getPrimeGenerator(it);
-        primes.resize(256);
-        it->primes = &primes[0];
       }
 
-      primeGenerator->fill(primes, &it->last_idx);
+      primeGenerator->fillNextPrimes(primes, &size);
 
       // There are 3 different cases here:
-      // 1) The primes array contains a few primes (<= 256).
+      // 1) The primes array contains a few primes (<= 512).
       //    In this case we return the primes to the user.
       // 2) The primes array is empty because the next
       //    prime > stop. In this case we reset the
@@ -114,59 +111,59 @@ void primesieve_generate_next_primes(primesieve_iterator* it)
       // 3) The next prime > 2^64. In this case the primes
       //    array contains an error code (UINT64_MAX) which
       //    is returned to the user.
-      if (it->last_idx == 0)
+      if (size == 0)
         clearPrimeGenerator(it);
-      else
-        break;
     }
   }
-  catch (exception&)
+  catch (const std::exception& e)
   {
+    std::cerr << "primesieve_iterator: " << e.what() << std::endl;
     clearPrimeGenerator(it);
-    primes.resize(1);
+    size = 1;
+    primes.resize(size);
     primes[0] = PRIMESIEVE_ERROR;
-    it->last_idx = 1;
     it->is_error = true;
     errno = EDOM;
   }
 
   it->i = 0;
-  it->last_idx--;
+  it->last_idx = size - 1;
+  it->primes = &primes[0];
 }
 
 void primesieve_generate_prev_primes(primesieve_iterator* it)
 {
   auto& primes = getPrimes(it);
+  std::size_t size = 0;
 
   try
   {
     if (it->primeGenerator)
       it->start = primes.front();
 
-    primes.clear();
     clearPrimeGenerator(it);
 
-    while (primes.empty())
+    while (!size)
     {
       IteratorHelper::prev(&it->start, &it->stop, it->stop_hint, &it->dist);
       it->primeGenerator = new PrimeGenerator(it->start, it->stop);
       auto primeGenerator = getPrimeGenerator(it);
-      if (it->start <= 2)
-        primes.push_back(0);
-      primeGenerator->fill(primes);
+      primeGenerator->fillPrevPrimes(primes, &size);
       clearPrimeGenerator(it);
     }
   }
-  catch (exception&)
+  catch (const std::exception& e)
   {
+    std::cerr << "primesieve_iterator: " << e.what() << std::endl;
     clearPrimeGenerator(it);
-    primes.resize(1);
+    size = 1;
+    primes.resize(size);
     primes[0] = PRIMESIEVE_ERROR;
     it->is_error = true;
     errno = EDOM;
   }
 
-  it->primes = &primes[0];
-  it->last_idx = primes.size() - 1;
+  it->last_idx = size - 1;
   it->i = it->last_idx;
+  it->primes = &primes[0];
 }
