@@ -3,7 +3,7 @@
 /// @brief  Count the primes within [10^i, 10^i + 10^8]
 ///         for i = 12 to 19
 ///
-/// Copyright (C) 2018 Kim Walisch, <kim.walisch@gmail.com>
+/// Copyright (C) 2022 Kim Walisch, <kim.walisch@gmail.com>
 ///
 /// This file is distributed under the BSD License. See the COPYING
 /// file in the top level directory.
@@ -18,10 +18,9 @@
 #include <iostream>
 #include <iomanip>
 
-using namespace std;
 using namespace primesieve;
 
-const array<uint64_t, 6> pix =
+const std::array<uint64_t, 6> pix =
 {
   3618282, // pi[10^12, 10^12+10^8]
   3342093, // pi[10^13, 10^13+10^8]
@@ -33,28 +32,28 @@ const array<uint64_t, 6> pix =
 
 void check(bool OK)
 {
-  cout << "   " << (OK ? "OK" : "ERROR") << "\n";
+  std::cout << "   " << (OK ? "OK" : "ERROR") << "\n";
   if (!OK)
-    exit(1);
+    std::exit(1);
 }
 
 int main()
 {
-  cout << left;
+  std::cout << std::left;
 
   for (size_t i = 0; i < pix.size(); i++)
   {
     size_t j = i + 12;
-    cout << "Sieving the primes within [10^" << j << ", 10^" << j << " + 10^8]" << endl;
-    uint64_t start = (uint64_t) pow(10.0, j);
+    std::cout << "Sieving the primes within [10^" << j << ", 10^" << j << " + 10^8]" << std::endl;
+    uint64_t start = (uint64_t) std::pow(10.0, j);
     uint64_t stop = start + (uint64_t) 1e8;
     uint64_t count = count_primes(start, stop);
-    cout << "\rPrime count: " << setw(7) << count;
+    std::cout << "\rPrime count: " << std::setw(7) << count;
     check(count == pix[i]);
   }
 
-  cout << endl;
-  cout << "All tests passed successfully!" << endl;
+  std::cout << std::endl;
+  std::cout << "All tests passed successfully!" << std::endl;
 
   return 0;
 }
