@@ -2,7 +2,7 @@
 /// @file   next_prime1.cpp
 /// @brief  Test next_prime() of primesieve::iterator.
 ///
-/// Copyright (C) 2017 Kim Walisch, <kim.walisch@gmail.com>
+/// Copyright (C) 2022 Kim Walisch, <kim.walisch@gmail.com>
 ///
 /// This file is distributed under the BSD License. See the COPYING
 /// file in the top level directory.
@@ -16,18 +16,16 @@
 #include <iostream>
 #include <vector>
 
-using namespace std;
-
 void check(bool OK)
 {
-  cout << "   " << (OK ? "OK" : "ERROR") << "\n";
+  std::cout << "   " << (OK ? "OK" : "ERROR") << "\n";
   if (!OK)
-    exit(1);
+    std::exit(1);
 }
 
 int main()
 {
-  vector<uint64_t> primes;
+  std::vector<uint64_t> primes;
   primesieve::generate_primes(100000, &primes);
   primesieve::iterator it;
   uint64_t stop = primes.size() - 1;
@@ -37,12 +35,12 @@ int main()
   {
     it.skipto(primes[i] - 1);
     prime = it.next_prime();
-    cout << "next_prime(" << primes[i] - 1 << ") = " << prime;
+    std::cout << "next_prime(" << primes[i] - 1 << ") = " << prime;
     check(prime == primes[i]);
 
     it.skipto(primes[i]);
     prime = it.next_prime();
-    cout << "next_prime(" << primes[i] << ") = " << prime;
+    std::cout << "next_prime(" << primes[i] << ") = " << prime;
     check(prime == primes[i + 1]);
   }
 
@@ -54,7 +52,7 @@ int main()
   for (; prime < 1000000000; prime = it.next_prime())
     sum += prime;
 
-  cout << "Sum of the primes below 10^9 = " << sum;
+  std::cout << "Sum of the primes below 10^9 = " << sum;
   check(sum == 24739512092254535ull);
 
   it.skipto(primes.back() - 200, primes.back());
@@ -68,12 +66,12 @@ int main()
     uint64_t old = prime;
     uint64_t p = primes[primes.size() - i];
     prime = it.prev_prime();
-    cout << "prev_prime(" << old << ") = " << prime;
+    std::cout << "prev_prime(" << old << ") = " << prime;
     check(prime == p);
   }
 
-  cout << endl;
-  cout << "All tests passed successfully!" << endl;
+  std::cout << std::endl;
+  std::cout << "All tests passed successfully!" << std::endl;
 
   return 0;
 }
