@@ -2,7 +2,7 @@
 /// @file   nth_prime2.cpp
 /// @brief  Test nth_prime edge cases
 ///
-/// Copyright (C) 2017 Kim Walisch, <kim.walisch@gmail.com>
+/// Copyright (C) 2022 Kim Walisch, <kim.walisch@gmail.com>
 ///
 /// This file is distributed under the BSD License. See the COPYING
 /// file in the top level directory.
@@ -15,14 +15,13 @@
 #include <exception>
 #include <cstdlib>
 
-using namespace std;
 using namespace primesieve;
 
 void check(bool OK)
 {
-  cout << "   " << (OK ? "OK" : "ERROR") << "\n";
+  std::cout << "   " << (OK ? "OK" : "ERROR") << "\n";
   if (!OK)
-    exit(1);
+    std::exit(1);
 }
 
 int main()
@@ -34,37 +33,37 @@ int main()
   n = 1;
   start = 1;
   res = nth_prime(n, start);
-  cout << "nth_prime(" << n << ", " << start << ") = " << res;
+  std::cout << "nth_prime(" << n << ", " << start << ") = " << res;
   check(res == 2);
 
   n = 1;
   start = 2;
   res = nth_prime(n, start);
-  cout << "nth_prime(" << n << ", " << start << ") = " << res;
+  std::cout << "nth_prime(" << n << ", " << start << ") = " << res;
   check(res == 3);
 
   n = -1;
   start = 102;
   res = nth_prime(n, start);
-  cout << "nth_prime(" << n << ", " << start << ") = " << res;
+  std::cout << "nth_prime(" << n << ", " << start << ") = " << res;
   check(res == 101);
 
   n = -1;
   start = 101;
   res = nth_prime(n, start);
-  cout << "nth_prime(" << n << ", " << start << ") = " << res;
+  std::cout << "nth_prime(" << n << ", " << start << ") = " << res;
   check(res == 97);
 
   n = -9592;
   start = 100000;
   res = nth_prime(n, start);
-  cout << "nth_prime(" << n << ", " << start << ") = " << res;
+  std::cout << "nth_prime(" << n << ", " << start << ") = " << res;
   check(res == 2);
 
   n = -9591;
   start = 100000;
   res = nth_prime(n, start);
-  cout << "nth_prime(" << n << ", " << start << ") = " << res;
+  std::cout << "nth_prime(" << n << ", " << start << ") = " << res;
   check(res == 3);
 
   try
@@ -72,12 +71,12 @@ int main()
     n = -1;
     start = 2;
     res = nth_prime(n, start);
-    cerr << "ERROR: nth_prime(" << n << ", " << start << ") = " << res;
+    std::cerr << "ERROR: nth_prime(" << n << ", " << start << ") = " << res;
     return 1;
   }
   catch (primesieve_error& e)
   {
-    cout << "OK: " << e.what() << endl;
+    std::cout << "OK: " << e.what() << std::endl;
   }
 
   try
@@ -85,16 +84,16 @@ int main()
     n = 1;
     start = 18446744073709551557ull;
     res = nth_prime(n, start);
-    cerr << "ERROR: nth_prime(" << n << ", " << start << ") = " << res;
+    std::cerr << "ERROR: nth_prime(" << n << ", " << start << ") = " << res;
     return 1;
   }
   catch (primesieve_error& e)
   {
-    cout << "OK: " << e.what() << endl;
+    std::cout << "OK: " << e.what() << std::endl;
   }
 
-  cout << endl;
-  cout << "All tests passed successfully!" << endl;
+  std::cout << std::endl;
+  std::cout << "All tests passed successfully!" << std::endl;
 
   return 0;
 }
