@@ -2,7 +2,7 @@
 /// @file   prev_prime1.cpp
 /// @brief  Test prev_prime() of primesieve::iterator.
 ///
-/// Copyright (C) 2017 Kim Walisch, <kim.walisch@gmail.com>
+/// Copyright (C) 2022 Kim Walisch, <kim.walisch@gmail.com>
 ///
 /// This file is distributed under the BSD License. See the COPYING
 /// file in the top level directory.
@@ -16,18 +16,16 @@
 #include <iostream>
 #include <vector>
 
-using namespace std;
-
 void check(bool OK)
 {
-  cout << "   " << (OK ? "OK" : "ERROR") << "\n";
+  std::cout << "   " << (OK ? "OK" : "ERROR") << "\n";
   if (!OK)
-    exit(1);
+    std::exit(1);
 }
 
 int main()
 {
-  vector<uint64_t> primes;
+  std::vector<uint64_t> primes;
   primesieve::generate_primes(100000, &primes);
   primesieve::iterator it;
   uint64_t back = primes.size() - 1;
@@ -37,12 +35,12 @@ int main()
   {
     it.skipto(primes[i] + 1);
     prime = it.prev_prime();
-    cout << "prev_prime(" << primes[i] + 1 << ") = " << prime;
+    std::cout << "prev_prime(" << primes[i] + 1 << ") = " << prime;
     check(prime == primes[i]);
 
     it.skipto(primes[i]);
     prime = it.prev_prime();
-    cout << "prev_prime(" << primes[i] << ") = " << prime;
+    std::cout << "prev_prime(" << primes[i] << ") = " << prime;
     check(prime == primes[i - 1]);
   }
 
@@ -54,13 +52,13 @@ int main()
   for (; prime > 0; prime = it.prev_prime())
     sum += prime;
 
-  cout << "Sum of the primes below 10^8 = " << sum;
+  std::cout << "Sum of the primes below 10^8 = " << sum;
   check(sum == 279209790387276ull);
 
   for (uint64_t i = 0; i < 1000; i++)
   {
     prime = it.prev_prime();
-    cout << "prev_prime(0) = " << prime;
+    std::cout << "prev_prime(0) = " << prime;
     check(prime == 0);
   }
 
@@ -68,7 +66,7 @@ int main()
   {
     uint64_t old = prime;
     prime = it.next_prime();
-    cout << "next_prime(" << old << ") = " << prime;
+    std::cout << "next_prime(" << old << ") = " << prime;
     check(prime == primes[i]);
   }
 
@@ -79,7 +77,7 @@ int main()
     prime = it.prev_prime();
     uint64_t p1 = primes.size() - (i + 1);
     uint64_t p2 = primes.size() - (i + 2);
-    cout << "prev_prime(" << primes[p1] << ") = " << prime;
+    std::cout << "prev_prime(" << primes[p1] << ") = " << prime;
     check(prime == primes[p2]);
   }
 
@@ -88,17 +86,17 @@ int main()
     uint64_t old = prime;
     uint64_t j = primes.size() - 1000 + i;
     prime = it.next_prime();
-    cout << "next_prime(" << old << ") = " << prime;
+    std::cout << "next_prime(" << old << ") = " << prime;
     check(prime == primes[j]);
   }
 
   it.skipto(18446744073709551615ull, 18446744073709551557ull);
   prime = it.prev_prime();
-  cout << "prev_prime(" << 18446744073709551615ull << ") = " << prime;
+  std::cout << "prev_prime(" << 18446744073709551615ull << ") = " << prime;
   check(prime == 18446744073709551557ull);
 
-  cout << endl;
-  cout << "All tests passed successfully!" << endl;
+  std::cout << std::endl;
+  std::cout << "All tests passed successfully!" << std::endl;
 
   return 0;
 }
